﻿package sqviewer 
{
	import flash.display.Graphics;
	import flash.display.Sprite;
	import flash.display.DisplayObjectContainer;
	import flash.text.TextFormat;
	import flash.text.TextField;
	import flash.display.Loader;
	import flash.events.Event;
	import flash.net.URLRequest;
	
	/**
	 * ...
	 * @author Kei Takahashi
	 */
	public class Button
	{
		private var container:DisplayObjectContainer;

		private var width:int = 0;
		private var height:int = 0;
		private var x:int = 1;
		private var y:int = 1;

		private var hilighted:Boolean = false;
		private var aniT:Number = 0;

		private var btnType:int;
		public static var BTN_NEXT:int = 10;
		public static var BTN_PREV:int = 11;
		public static var BTN_GO_BACK:int = 12;
		public static var BTN_PLAY:int = 13;
		public static var BTN_STOP:int = 14;
		
		private var origAlpha:Number = 0;
		private var destAlpha:Number = 0;
		private var alpha:Number = 0;
		
		private var btn_back_url:String = "back.png";
		private var btn_stop_url:String = "stop.png";
		private var btn_play_url:String = "play.png";
		private var loader:Loader;  
		private var loadFinished:Boolean = false;  
		
		private var s:Sprite = new Sprite();
		public function Button(container:DisplayObjectContainer, btnType:int) 
		{
			this.container = container;
			this.btnType = btnType;
			this.container.addChild(this.s);
			
			if (btnType == BTN_GO_BACK) {
				beginLoad(btn_back_url);
			} else if(btnType == BTN_STOP) {
				beginLoad(btn_stop_url);
			} else if(btnType == BTN_PLAY) {
				beginLoad(btn_play_url);
			} 
		}
		
		
		private function beginLoad(s:String):void
        {  
            loader = new Loader();  
            loader.load(new URLRequest(s));  
            loader.contentLoaderInfo.addEventListener(Event.COMPLETE, afterLoad);  
        }

		private function afterLoad(event:Event):void
		{
			loader.visible = false;
			loader.alpha = 0.0;
			s.addChild(loader);
			loadFinished = true;
			resizeGoBackIcon();
		}

		
		public function resize(x:int, y:int, width:int, height:int) :void
		{
			this.x = x;
			this.y = y;
			this.width = width;
			this.height = height;
			switch(btnType) {
				case BTN_GO_BACK:
				case BTN_STOP:
				case BTN_PLAY:{
					resizeGoBackIcon();
					break;
				}
			}
		}

		private function resizeGoBackIcon():void {
			if(loadFinished){
				loader.x = x;
				loader.y = y;
				loader.width = this.width;
				loader.height = this.width;
			}
		}
		
		
		public function onFrame():void 
		{
			updatePos();
			if (alpha <= 0.0) return;
			switch(btnType) {
				case BTN_PREV:
				case BTN_NEXT:
					drawArrow();
					break;
				case BTN_GO_BACK:
				case BTN_PLAY:
				case BTN_STOP:
					drawGoBack();
					break;
			}
		}

		
		private function updatePos():void {
			var t:Number = 1 - (1 - aniT) * (1 - aniT) * (1 - aniT);
			if (t > 1.0) t = 1.0;
			alpha = origAlpha * (1 - t) + destAlpha * t;
			aniT += 0.1;
			if (aniT < 1.0) return;
		}
		
		private function drawArrow():void {
			s.alpha = alpha;
			s.graphics.clear();
			if (hilighted) {
				s.graphics.lineStyle(4.0, 0xffffff, 0.6 );
				s.graphics.beginFill(0xffffff, 0.3);
			} else {
				s.graphics.lineStyle(4.0, 0xffffff, 0.5 );
			}
			s.graphics.drawRoundRect(x, y, width, height, width / 2);
			if (hilighted) {
				s.graphics.endFill();
			}
			
			var rx0:Number = (btnType == BTN_PREV) ? 0.2 : 0.8;
			var rx1:Number = (btnType == BTN_PREV) ? 0.8 : 0.2;
			
			s.graphics.lineStyle();
			if (hilighted) {
				s.graphics.beginFill(0xffffff, 0.6);
			} else {
				s.graphics.beginFill(0xffffff, 0.5);
			}
			
			s.graphics.moveTo(x + width * rx0, y + height * 0.5);
			s.graphics.lineTo(x + width * rx1, y + height * 0.4);
			s.graphics.lineTo(x + width * rx1, y + height * 0.6);
			s.graphics.lineTo(x + width * rx0, y + height * 0.5);
			s.graphics.endFill();
		}

		
		
		private function drawGoBack():void {
			s.graphics.clear();
			s.graphics.lineStyle();
			if (hilighted) {
				s.graphics.lineStyle(4.0, 0xffffff, 0.6 );
				s.graphics.beginFill(0xffffff, 0.3);
			} else {
				s.graphics.lineStyle(4.0, 0xffffff, 0.5);
			}
			s.graphics.drawRoundRect(x, y, width, height, width / 4);
			if (hilighted) {
				s.graphics.endFill();
			}
			
			if (! loadFinished) return;
			if (hilighted) {
				loader.alpha = s.alpha * 0.8;
			} else {
				loader.alpha = s.alpha * 0.6;
			}
		}
		

		public function myHitTest(mouseX:Number, mouseY:Number):Boolean {
			if (s.visible == false) return false;
			if (alpha <= 0.0) return false;
			if (mouseX < x) return false;
			if (x + width < mouseX) return false;
			if (mouseY < y) return false;
			if (y + height < mouseY) return false;
			return true;
		}

		public function show():void {
			//container.addChild(this.s);
			origAlpha = alpha;
			destAlpha = 1.0;
			aniT = 0.0;
			s.visible = true;
			switch(btnType) {
				case BTN_GO_BACK:
				case BTN_STOP:
				case BTN_PLAY:{
					loader.visible = true;
					break;
				}
			}
		}
	
		public function hide():void {
			//container.removeChild(this.s);
			origAlpha = alpha;
			destAlpha = 0.0;
			aniT = 0.0;
			s.visible = false;
			switch(btnType) {
				case BTN_GO_BACK:
				case BTN_STOP:
				case BTN_PLAY:{
					loader.visible = false;
					break;
				}
			}
		}
		
		
		public function hilight():void {
			this.hilighted = true;
		}

		public function unhilight():void {
			this.hilighted = false;
		}
	}

}