﻿package sqviewer 
{
	/**
	 * ...
	 * @author Kei Takahashi
	 */
	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
    import flash.net.URLRequest;  
	import flash.ui.Mouse;
	import flash.utils.Endian;
	import sqviewer.Entry;
	import flash.events.MouseEvent;
	import sqviewer.Button;
	
	public class Group
	{
		private var entries:Array = new Array();
		private var container:DisplayObjectContainer;
		private var focusedIndex:int = -1;
		private var order:Order;

		// btns
		private var nextBtn:Button;
		private var prevBtn:Button;
		private var goBackBtn:Button;
		private var playBtn:Button;
		private var stopBtn:Button;

		// idle management
		private var idleTime:int = 0;
		private var slideShowTime:int = 0;
		private var idleFlag:Boolean;
		
		private var stopFlag:Boolean;
		
		private var mode:int;
		private var MODE_LIST:int = 10;
		private var MODE_FOCUS_ONE:int = 11;
		private var MODE_FULL:int = 12;
		
		
		public function Group(container:DisplayObjectContainer, urls:Array)
        {  
			this.container = container;
			container.addEventListener(MouseEvent.MOUSE_MOVE, onMove);
			container.addEventListener(Event.ENTER_FRAME, onFrame);
			container.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			
			for (var i:int = 0; i < urls.length; i++) {
				var e:Entry = new Entry(container, urls[i], i);
				entries.push(e);
			}
			
			prevBtn = new Button(container, Button.BTN_PREV); 
 			nextBtn = new Button(container, Button.BTN_NEXT); 
 			goBackBtn = new Button(container, Button.BTN_GO_BACK); 

 			playBtn = new Button(container, Button.BTN_PLAY);
 			stopBtn = new Button(container, Button.BTN_STOP); 

			idleFlag = false;
			order = new Order(entries.length);
			mode = MODE_LIST;
			stopFlag = false;
        }  
		
		
		public function onMove(event:MouseEvent):void {
			endIdleMode();
			var mouseX:Number = event.stageX;
			var mouseY:Number = event.stageY;

			if (mode == MODE_FULL) {
				if(prevBtn.myHitTest(mouseX, mouseY)){
					prevBtn.hilight();
				} else {
					prevBtn.unhilight();
				}
				
				if(nextBtn.myHitTest(mouseX, mouseY)){
					nextBtn.hilight();
				} else {
					nextBtn.unhilight();
				}
				
				if(goBackBtn.myHitTest(mouseX, mouseY)){
					goBackBtn.hilight();
				} else {
					goBackBtn.unhilight();
				}
					
				if (!stopFlag) {
					if(stopBtn.myHitTest(mouseX, mouseY)){
						stopBtn.hilight();
					} else {
						stopBtn.unhilight();
					}
				} else {
					if(playBtn.myHitTest(mouseX, mouseY)){
						playBtn.hilight();
					} else {
						playBtn.unhilight();
					}
				}
				
				return;
			}

			for (var i :int = 0; i < entries.length; i++) {
				var index:int = order.get(i);
				var e:Entry = entries[index];
				if (e.myHitTest(mouseX, mouseY)) {
					focusOn(index);
					return;
				}
			}
			toListMode();
		}

		private function beginIdleMode():void {
			idleFlag = true;
			Mouse.hide();
			if (mode == MODE_FULL) {
				prevBtn.hide();
				nextBtn.hide();
				goBackBtn.hide();
				playBtn.hide();
				stopBtn.hide();
			}
		}

		
		private function endIdleMode():void {
			idleFlag = false;
			idleTime = 0;
			Mouse.show();
			if (mode == MODE_FULL) {
				prevBtn.show();
				nextBtn.show();
				goBackBtn.show();
				if (stopFlag) {
					playBtn.show();
					stopBtn.hide();
				} else {
					stopBtn.show();
					playBtn.hide();
				}
			}
		}
		

		public function onMouseDown(event:MouseEvent):void {
			endIdleMode();
			var mouseX:Number = event.stageX;
			var mouseY:Number = event.stageY;
			if (mode == MODE_FULL) {
				if (prevBtn.myHitTest(mouseX, mouseY)) {
					// TODO: whiten btn
					gobackMaximized();
				} else if(nextBtn.myHitTest(mouseX, mouseY)){
					proceedMaximized();
				} else if(goBackBtn.myHitTest(mouseX, mouseY)){
					focusOn(focusedIndex);
				} else if ((!stopFlag) && stopBtn.myHitTest(mouseX, mouseY)) {
						stopFlag = true;
						stopBtn.hide();
						playBtn.show();
				} else if (stopFlag && playBtn.myHitTest(mouseX, mouseY)) {
					stopFlag = false;
					beginIdleMode();
					/*
					playBtn.hide();
					stopBtn.show();
					*/
				} else {
					focusOn(focusedIndex);
				}
			} else if (mode == MODE_LIST) {
				//TODO: check if buttons are not clicked
				
			} else if (mode == MODE_FOCUS_ONE) {
				var e:Entry = entries[focusedIndex];
				if (e.myHitTest(mouseX, mouseY)) {
					maximize(focusedIndex);
				} else {
					toListMode();
				}
			}
		}

		public function onFrame(event:Event):void {
			for (var i:int = entries.length - 1; i >= 0; i--) {	
				var e:Entry = entries[order.get(i)];
				e.onFrame(event);
			}
			
			prevBtn.onFrame();
			nextBtn.onFrame();
			goBackBtn.onFrame();
			playBtn.onFrame();
			stopBtn.onFrame();
			
			if (!idleFlag){
				idleTime++;
				if(idleTime >= Env.IDLE_MAX) {
					beginIdleMode();
					
					if (mode == MODE_FULL) {
						slideShowTime = Env.SLIDESHOW_DURATION - 1;
					}
				}
			} else {
				if (mode == MODE_FULL) {
					if(stopFlag) {
					} else if (slideShowTime++ > Env.SLIDESHOW_DURATION) {
						slideShowTime = 0;
						proceedMaximized();
					}
				} else if(mode == MODE_FOCUS_ONE) {
					toListMode();
				}
				
			}
		}

		private function proceedMaximized():void {
			var newFocused:int = focusedIndex + 1;
			if (newFocused == entries.length){
				newFocused = 0;
			}
			maximize(newFocused);
		}
		

		private function gobackMaximized():void {
			var newFocused:int = focusedIndex - 1;
			if (newFocused == -1){
				newFocused = entries.length - 1;
			}
			maximize(newFocused);
		}

		public function focusOn(entryIndex:int):void
        {
			prevBtn.hide();
			nextBtn.hide();
			goBackBtn.hide();
			playBtn.hide();
			stopBtn.hide();
			
			if (mode == MODE_FOCUS_ONE && 
			  this.focusedIndex == entryIndex)
				return;
			mode = MODE_FOCUS_ONE;
			if(this.focusedIndex != -1){
				var e:Entry = entries[this.focusedIndex];
				e.focusOut();
			}
			this.focusedIndex = entryIndex;
			if (this.focusedIndex == -1) return;

			order.toTop(entryIndex, entries);
			entries[this.focusedIndex].focusOn();
		}

		
		public function toListMode():void
        {  
			mode = MODE_LIST;
			if(this.focusedIndex != -1){
				var e:Entry = entries[this.focusedIndex];
				e.focusOut();
			}
			focusedIndex = -1;
			// may do : order.reset();
		}

		public function maximize(entryIndex:int):void
        {  
			if (mode == MODE_FULL) {
				// fade out -> fade in
				if (focusedIndex == entryIndex) return;
				var e:Entry = entries[focusedIndex];
				e.unmaximizeFadeOut();
				focusedIndex = entryIndex;
				e = entries[entryIndex];
				e.maximizeFadeIn();
				order.toTop(focusedIndex, entries);
			} else {
				mode = MODE_FULL;
				stopFlag = true;
				this.focusedIndex = entryIndex;
				order.toTop(focusedIndex, entries);
				entries[focusedIndex].maximize();
				
			}
			
			if (!idleFlag) {
				prevBtn.show();
				nextBtn.show();
				goBackBtn.show();
				if(stopFlag){
					playBtn.show();
					stopBtn.hide();
				} else {
					playBtn.hide();
					stopBtn.show();
				}
			}
		}
		
		
		public function resize():void {
			prevBtn.resize(Env.W * 0.01, Env.H * 0.3, Env.W * 0.04, Env.H * 0.4);
			nextBtn.resize(Env.W * 0.95, Env.H * 0.3, Env.W * 0.04, Env.H * 0.4);
			goBackBtn.resize(Env.W * 0.05, Env.H * 0.88, (int)(0.1 * Env.H), (int)(0.1 * Env.H));
			stopBtn.resize(Env.W * 0.15, Env.H * 0.88, (int)(0.1 * Env.H), (int)(0.1 * Env.H));
			playBtn.resize(Env.W * 0.15, Env.H * 0.88, (int)(0.1 * Env.H), (int)(0.1 * Env.H));
			for each (var e:Entry in entries) {
				e.resize();
			}
		}
	}
}